/*
 * MosP - Mind Open Source Project    http://www.mosp.jp/
 * Copyright (C) MIND Co., Ltd.       http://www.e-mind.co.jp/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jp.mosp.kintai.workflow.action;

import java.util.Date;

import jp.mosp.common.CommonConst;
import jp.mosp.common.common.BaseVo;
import jp.mosp.common.common.MospConst;
import jp.mosp.common.common.MospException;
import jp.mosp.common.common.MospUtility;
import jp.mosp.common.utils.DateUtil;
import jp.mosp.common.utils.StringUtil;
import jp.mosp.kintai.common.action.MosPWorkflowAction;
import jp.mosp.kintai.common.dto.CtWorkflowDto;
import jp.mosp.kintai.common.part.KintaiApprovalPart;
import jp.mosp.kintai.common.part.WorkflowPart;
import jp.mosp.kintai.workflow.dto.CtOvertimeRequestDto;
import jp.mosp.kintai.workflow.vo.OverTimeRequestCardVo;

/**
 * @author yoshida
 *
 */
public class OverTimeRequestCardAction extends MosPWorkflowAction {
	
	// コマンド
	private static final String	CMD_CARD_SHOW	= "K1415";
	private static final String	CMD_CARD_SELECT	= "K1416";
	private static final String	CMD_CARD_INSERT	= "K1417";
	private static final String	CMD_CARD_UPDATE	= "K1418";
	private static final String	CMD_CARD_CANCEL	= "K1419";
	

	/**
	 * コンストラクタ
	 */
	public OverTimeRequestCardAction() {
		super();
	}
	
	public void action() throws Exception {
		// VOの初期化
		prepareVo(false);
		// 初期処理
		init();
		// コマンド毎の処理
		if (cmd.equals(CMD_CARD_SHOW)) {
			show();
		} else if (cmd.equals(CMD_CARD_SELECT)) {
			// 表示(キーで情報を検索)
			select();
		} else if (cmd.equals(CMD_CARD_INSERT)) {
			// 新規登録
			insert();
		} else if (cmd.equals(CMD_CARD_UPDATE)) {
			// 更新
			update();
		} else if (cmd.equals(CMD_CARD_CANCEL)) {
			// 取り下げ
			cancel();
		} else {
			throw new MospException(MospConst.EX_CMD_INVALID);
		}
	}
	
	@Override
	protected BaseVo getSpecificVo() {
		return new OverTimeRequestCardVo();
	}
	
	@Override
	protected void init() throws Exception {
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		// ユーザ情報の取得
		getInfo(user.getUserId());
		// ユーザ情報の表示
		setLabelBasisInfo(userBasis);
		// プルダウンのセット
		vo.aryPltYear = StringUtil.getYearArray(MospUtility.getYear(DateUtil.getSystemDate()));
		vo.aryPltMonth = StringUtil.getMonthArray();
		vo.aryPltDay = StringUtil.getDayArray();
		vo.aryPltHour = StringUtil.getCountArray(0, 24, 1);
		// 入力単位時間設定
		setInputUnit();
		final int end = 59;
		vo.aryPltMin = StringUtil.getCountArray(0, end, inputUnit);
		vo.setParams(request);
	}
	
	@Override
	protected void show() throws Exception {
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		// 初期値の設定
		vo.setDefaultValues();
		Date todayDate = DateUtil.getSystemDate();
		vo.setPltRequestYear(String.valueOf(MospUtility.getYear(todayDate)));
		vo.setPltRequestMonth(String.valueOf(MospUtility.getMonth(todayDate)));
		vo.setPltRequestDay(String.valueOf(MospUtility.getDay(todayDate)));
		// モード設定
		vo.setMode(OverTimeRequestCardVo.MODE_INSERT);
	}
	
	@Override
	protected void select() throws Exception {
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		// データの取得
		CtOvertimeRequestDto dto = getOverTimeRequest(vo.getIntParam(key1));
		chkExist(dto);
		// VOに情報をセット
		vo.setFields(dto);
		// 承認状況の設定
		setApprovalStatus(dto);
		// セッションにDTOをセット
		setSessionDto(dto);
	}
	
	@Override
	protected void insert() throws Exception {
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		// パラメーターの妥当性確認
		vo.validate();
		// DTOの準備
		CtOvertimeRequestDto dto = new CtOvertimeRequestDto();
		vo.setDtoFields(dto);
		dto.setKCode(userBasis.getKCode());
		dto.setRequestDate(DateUtil.getSystemDate());
		// 重複確認
		chkDuplicate(getOverTimeRequest(dto.getKCode(), dto.getWorkDate()));
		// テーブルロック準備
		prepareTableLock();
		// 対象テーブル追加
		addTargetTable(overTimeDao(), true);
		// ワークフロー登録
		long workflow = requestRegist(userBasis.getKCode(), WorkflowPart.isDraft(key1));
		if (workflow != 0) {
			dto.setWorkflow(workflow);
			// 残業申請
			String maxVaoucher = overTimeDao().findForMax();
			if (StringUtil.isNull(maxVaoucher)) {
				maxVaoucher = "-1";
			} else {
				overTimeDao().findForUpdate(Integer.parseInt(maxVaoucher));
			}
			// 最大値をセット
			dto.setVoucher(Integer.parseInt(maxVaoucher) + 1);
			overTimeDao().insert(dto);
			chkInsert(getOverTimeRequest(dto.getVoucher()));
			// コミット
			commit();
			// DTOの再取得
			dto = getOverTimeRequest(dto.getVoucher());
			vo.setHidVoucher(String.valueOf(dto.getVoucher()));
			// メッセージ設定
			setMessage(getMessage(CommonConst.MSG_CODE_REQUEST_INSERT, ""));
		} else {
			StringBuffer sb = new StringBuffer();
			sb.append(DateUtil.convDateToFormatDateJP(dto.getWorkDate()));
			sb.append(CommonConst.MSG_OF);
			sb.append(CommonConst.NAM_OVER_TIME_REQUEST);
			// エラーメッセージ設定
			addErrMessage(getMessage(CommonConst.MSG_CODE_SELECT_CANNOT_INSERT, sb.toString()));
		}
		// テーブルロック解除
		unLockTables();
		// DTOの表示
		vo.setFields(dto);
		// 承認状況の設定
		setApprovalStatus(dto);
		// セッションにDTOをセット
		setSessionDto(dto);
	}
	
	@Override
	protected void update() throws Exception {
		// DTOの取得
		CtOvertimeRequestDto dto = (CtOvertimeRequestDto)getSessionDto();
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		vo.setPltRequestYear(String.valueOf(MospUtility.getYear(dto.getWorkDate())));
		vo.setPltRequestMonth(String.valueOf(MospUtility.getMonth(dto.getWorkDate())));
		vo.setPltRequestDay(String.valueOf(MospUtility.getDay(dto.getWorkDate())));
		// パラメーターの妥当性確認
		vo.validate();
		// DTOの準備
		vo.setDtoFields(dto);
		dto.setKCode(userBasis.getKCode());
		dto.setRequestDate(DateUtil.getSystemDate());
		// 排他確認
		chkExclusive(dto, overTimeDao().findForUpdate(dto.getVoucher()));
		// 更新
		overTimeDao().update(dto);
		// 更新確認
		chkUpdate(getOverTimeRequest(dto.getVoucher()));
		boolean canCommit = true;
		// 申請更新の場合
		if (!WorkflowPart.isDraft(key1)) {
			// ワークフロー取得
			CtWorkflowDto workflowDto = getWorkflowDto(dto.getWorkflow());
			// 承認状況の変更
			workflowDto.setCurrentStatus(WorkflowPart.getStatusCodeArray()[0]);
			// 勤怠承認コード
			String statusCode = toKintaiApprovalStatus(dto.getWorkflow());
			// 差戻→申請中の場合
			if (KintaiApprovalPart.is1stReturned(statusCode) || KintaiApprovalPart.is2ndReturned(statusCode)) {
				// 申請中へ移行
				workflowDto.setLayerNumber(CommonConst.NUM_REQUEST_LAYER);
			}
			// ワークフローの更新
			canCommit = workflowUpdate(workflowDto);
		}
		if (canCommit) {
			// コミット
			commit();
			// メッセージ設定
			setMessage(getMessage(MospConst.MSG_UPDATE, ""));
		}
		// DTOの再取得
		CtOvertimeRequestDto retDto = getOverTimeRequest(dto.getVoucher());
		chkExist(retDto);
		// 承認状況の設定
		setApprovalStatus(retDto);
		// セッションにDTOをセット
		setSessionDto(retDto);
	}
	
	/**
	 * 取り下げ処理
	 * @throws Exception	例外処理が発生した場合
	 */
	private void cancel() throws Exception {
		// DTOの取得
		CtOvertimeRequestDto dto = (CtOvertimeRequestDto)getSessionDto();
		// 承認済みの場合、取り下げ不可
		if (!isStatusApproved(dto.getWorkflow())) {
			// 論理削除
			dto.setDeleteFlag(MospConst.DELETE_FLAG_ON);
			overTimeDao().update(dto);
			// ワークフロー
			CtWorkflowDto workflowDto = getWorkflowDto(dto.getWorkflow());
			// 論理削除
			workflowDto.setDeleteFlag(MospConst.DELETE_FLAG_ON);
			workflowUpdate(workflowDto);
			// コミット
			commit();
			// 一覧へ遷移
			setNextCmd(OverTimeRequestListAction.CMD_LIST_RE_SEARCH);
			// メッセージをセット
			setMessage(getMessage(CommonConst.MSG_CODE_REQUEST_WITHDRAW, ""));
		} else {
			// エラーメッセージ
			addAlreadyApprovedMessage(dto.getKCode(), dto.getWorkDate(), CommonConst.NAM_OVER_TIME_REQUEST);
		}
	}
	
	/**
	 * 承認状況情報の設定
	 * @param dto			対象DTO
	 * @throws Exception	例外処理が発生した場合
	 */
	private void setApprovalStatus(CtOvertimeRequestDto dto) throws Exception {
		OverTimeRequestCardVo vo = (OverTimeRequestCardVo)getVo();
		// 勤怠承認ステータスコード
		String statusCode = toKintaiApprovalStatus(dto.getWorkflow());
		vo.setLblStatus(getApprovalName(statusCode));
		if (isStatusDraft(dto.getWorkflow())) {
			// 下書き
			vo.setMode(OverTimeRequestCardVo.MODE_UPDATE);
		} else if (KintaiApprovalPart.isRequest(statusCode)) {
			// 申請中
			vo.setMode(OverTimeRequestCardVo.MODE_REQUEST);
		} else if (KintaiApprovalPart.is1stApproved(statusCode)) {
			// 一次承認
			vo.setLblApprover(getKName(getWorkflowDto(dto.getWorkflow()).getUpdateUser()));
			vo.setMode(OverTimeRequestCardVo.MODE_APPROVED);
		} else if (KintaiApprovalPart.is1stReturned(statusCode)) {
			// 一次差戻
			vo.setLblApprover(getKName(getWorkflowDto(dto.getWorkflow()).getUpdateUser()));
			// 最新コメントの取得
			vo.setLblComment(getCurrentWorkflowComment(userBasis.getKCode(), dto.getWorkflow()));
			vo.setMode(OverTimeRequestCardVo.MODE_RETURNED);
		} else if (KintaiApprovalPart.is2ndApproved(statusCode)) {
			// 二次承認
			vo.setLblApprover(getKName(getWorkflowDto(dto.getWorkflow()).getUpdateUser()));
			vo.setMode(OverTimeRequestCardVo.MODE_APPROVED);
		} else if (KintaiApprovalPart.is2ndReturned(statusCode)) {
			// 二次差戻
			vo.setLblApprover(getKName(getWorkflowDto(dto.getWorkflow()).getUpdateUser()));
			// 最新コメントの取得
			vo.setLblComment(getCurrentWorkflowComment(userBasis.getKCode(), dto.getWorkflow()));
			vo.setMode(OverTimeRequestCardVo.MODE_RETURNED);
		}
	}
	
}
